import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { activePluginsDataSelector } from '@/redux/plugins/plugins.selectors';
import { PluginsManager } from '@/services/pluginsManager';
import axios from 'axios';
import { ChangeEvent, useMemo, useState } from 'react';

type UseLoadPluginReturnType = {
  handleChangePluginUrl: (event: ChangeEvent<HTMLInputElement>) => void;
  handleLoadPlugin: () => Promise<void>;
  isPending: boolean;
  pluginUrl: string;
};

export const useLoadPluginFromUrl = (): UseLoadPluginReturnType => {
  const [pluginUrl, setPluginUrl] = useState('');
  const [isLoading, setIsLoading] = useState(false);
  const activePlugins = useAppSelector(activePluginsDataSelector);

  const isPending = useMemo(
    () => !pluginUrl || isLoading || !URL.canParse(pluginUrl),
    [pluginUrl, isLoading],
  );

  const handleLoadPlugin = async (): Promise<void> => {
    try {
      setIsLoading(true);
      const response = await axios(pluginUrl);
      const pluginScript = response.data;

      /* eslint-disable no-new-func */
      const loadPlugin = new Function(pluginScript);

      const hash = PluginsManager.setHashedPlugin({
        pluginUrl,
        pluginScript,
      });

      if (!(hash in activePlugins)) {
        loadPlugin();
      }

      setPluginUrl('');
    } finally {
      setIsLoading(false);
    }
  };
  const handleChangePluginUrl = (event: ChangeEvent<HTMLInputElement>): void => {
    setPluginUrl(event.target.value);
  };

  return {
    handleChangePluginUrl,
    handleLoadPlugin,
    isPending,
    pluginUrl,
  };
};
