import {
  Accordion,
  AccordionItem,
  AccordionItemButton,
  AccordionItemHeading,
  AccordionItemPanel,
} from '@/shared/Accordion';
import { PinItem, PinTypeWithNone } from '../PinsList/PinsList.types';
import {
  getAdditionalInfo,
  getEntityDescriptions,
  getEntityNames,
  getEntitySynonyms,
} from './AccordionsDetails.utils';

interface AccordionsDetailsProps {
  pinsList: PinItem[];
  type: PinTypeWithNone;
}

export const AccordionsDetails = ({ pinsList, type }: AccordionsDetailsProps): JSX.Element => {
  return (
    <div data-testid="accordions-details">
      <Accordion allowZeroExpanded className="px-6">
        <AccordionItem>
          <AccordionItemHeading>
            <AccordionItemButton className="capitalize">{type}</AccordionItemButton>
          </AccordionItemHeading>
          <AccordionItemPanel>{getEntityNames(pinsList)}</AccordionItemPanel>
        </AccordionItem>
        <AccordionItem>
          <AccordionItemHeading>
            <AccordionItemButton>Description</AccordionItemButton>
          </AccordionItemHeading>
          <AccordionItemPanel>
            <div data-testid="details-description">{getEntityDescriptions(pinsList)}</div>
          </AccordionItemPanel>
        </AccordionItem>
        <AccordionItem>
          <AccordionItemHeading>
            <AccordionItemButton>Synonyms</AccordionItemButton>
          </AccordionItemHeading>
          <AccordionItemPanel>{getEntitySynonyms(pinsList)}</AccordionItemPanel>
        </AccordionItem>
      </Accordion>
      {type === 'drugs' && (
        <div className="flex justify-between px-6 py-4 text-sm font-bold">
          <div>Blood brain barrier</div>
          <div>{getAdditionalInfo(pinsList, type)}</div>
        </div>
      )}
      {type === 'chemicals' && (
        <div className="flex justify-between px-6 py-4 text-sm">
          <div className="font-bold">Direct Evidence Publications</div>
          <div>{getAdditionalInfo(pinsList, type)}</div>
        </div>
      )}
    </div>
  );
};
