import { assertNever } from '@/utils/assertNever';
import { AccordionsDetails } from '../AccordionsDetails/AccordionsDetails.component';
import { PinItem, PinTypeWithNone } from './PinsList.types';
import { PinsListItem } from './PinsListItem';

interface PinsListProps {
  pinsList: PinItem[];
  type: PinTypeWithNone;
}

export const PinsList = ({ pinsList, type }: PinsListProps): JSX.Element => {
  switch (type) {
    case 'drugs':
      return (
        <div className="h-[calc(100%-214px)] max-h-[calc(100%-214px)] overflow-auto">
          <AccordionsDetails pinsList={pinsList} type={type} />
          <ul className="px-6 py-2" data-testid="pins-list">
            {pinsList.map(result => {
              return result.data.targets.map(pin => (
                <PinsListItem key={pin.name} name={pin.name} type={type} pin={pin} />
              ));
            })}
          </ul>
        </div>
      );
    case 'bioEntity':
      return <div />;
    case 'chemicals':
      return (
        <div className="h-[calc(100%-214px)] max-h-[calc(100%-214px)] overflow-auto">
          <AccordionsDetails pinsList={pinsList} type={type} />
          <ul className="px-6 py-2" data-testid="pins-list">
            {pinsList.map(result => {
              return result.data.targets.map(pin => (
                <PinsListItem key={pin.name} name={pin.name} type={type} pin={pin} />
              ));
            })}
          </ul>
        </div>
      );
    case 'none':
      return <div />;
    default:
      return assertNever(type);
  }
};
