/* eslint-disable no-magic-numbers */
import { bioEntitiesContentFixture } from '@/models/fixtures/bioEntityContentsFixture';
import { chemicalsFixture } from '@/models/fixtures/chemicalsFixture';
import { drugsFixture } from '@/models/fixtures/drugFixtures';
import { StoreType } from '@/redux/store';
import { PinDetailsItem } from '@/types/models';
import {
  InitialStoreState,
  getReduxWrapperWithStore,
} from '@/utils/testing/getReduxWrapperWithStore';
import { render, screen } from '@testing-library/react';
// import { MODELS_MOCK_SHORT } from '@/models/mocks/modelsMock';
import { act } from 'react-dom/test-utils';
import { PinTypeWithNone } from '../PinsList.types';
import { PinsListItem } from './PinsListItem.component';
import { useVisiblePinsPolygonCoordinates } from './hooks/useVisiblePinsPolygonCoordinates';

const setBounds = jest.fn();

setBounds.mockImplementation(() => {});
jest.mock('../../../../../../../utils/map/useSetBounds', () => ({
  _esModule: true,
  useSetBounds: (): jest.Mock => setBounds,
}));

const useVisiblePinsPolygonCoordinatesMock = useVisiblePinsPolygonCoordinates as jest.Mock;

jest.mock('./hooks/useVisiblePinsPolygonCoordinates', () => ({
  _esModule: true,
  useVisiblePinsPolygonCoordinates: jest.fn(),
}));

setBounds.mockImplementation(() => {});

const DRUGS_PIN = {
  name: drugsFixture[0].targets[0].name,
  pin: drugsFixture[0].targets[0],
};

const CHEMICALS_PIN = {
  name: chemicalsFixture[0].targets[0].name,
  pin: chemicalsFixture[0].targets[0],
};

const renderComponent = (
  name: string,
  pin: PinDetailsItem,
  type: PinTypeWithNone,
  initialStoreState: InitialStoreState = {},
): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <PinsListItem name={name} type={type} pin={pin} />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('PinsListItem - component ', () => {
  it('should display full name of pin', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const drugName = drugsFixture[0].targets[0].name;

    expect(screen.getByText(drugName)).toBeInTheDocument();
  });
  it('should display list of elements for pin for drugs', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const firstPinElementType = drugsFixture[0].targets[0].targetParticipants[0].type;
    const firstPinElementResource = drugsFixture[0].targets[0].targetParticipants[0].resource;
    const secondPinElementType = drugsFixture[0].targets[0].targetParticipants[1].type;
    const secondPinElementResource = drugsFixture[0].targets[0].targetParticipants[1].resource;

    expect(screen.getByText(firstPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(firstPinElementResource, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementResource, { exact: false })).toBeInTheDocument();
  });
  it('should display list of references for pin', () => {
    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const firstPinReferenceType = drugsFixture[0].targets[0].references[0].type;
    const firstPinReferenceResource = drugsFixture[0].targets[0].references[0].resource;
    const secondPinReferenceType = drugsFixture[0].targets[0].references[1].type;
    const secondPinReferenceResource = drugsFixture[0].targets[0].references[1].resource;

    expect(screen.getByText(firstPinReferenceType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(firstPinReferenceResource, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinReferenceType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinReferenceResource, { exact: false })).toBeInTheDocument();
  });
  it('should display list of elements for pin for chemicals', () => {
    renderComponent(CHEMICALS_PIN.name, CHEMICALS_PIN.pin, 'chemicals');

    const firstPinElementType = chemicalsFixture[0].targets[0].targetParticipants[0].type;
    const firstPinElementResource = chemicalsFixture[0].targets[0].targetParticipants[0].resource;
    const secondPinElementType = chemicalsFixture[0].targets[0].targetParticipants[1].type;
    const secondPinElementResource = chemicalsFixture[0].targets[0].targetParticipants[1].resource;

    expect(screen.getByText(firstPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(firstPinElementResource, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementType, { exact: false })).toBeInTheDocument();
    expect(screen.getByText(secondPinElementResource, { exact: false })).toBeInTheDocument();
  });

  // TODO - it's probably flacky test
  it.skip('should not display list of elements for pin for bioentities', () => {
    renderComponent(CHEMICALS_PIN.name, CHEMICALS_PIN.pin, 'drugs');

    const bioEntityName = bioEntitiesContentFixture[2].bioEntity.fullName
      ? bioEntitiesContentFixture[2].bioEntity.fullName
      : '';

    expect(screen.queryByText(bioEntityName, { exact: false })).not.toBeInTheDocument();
  });
  it("should not display list of available submaps for pin when there aren't any submaps", () => {
    const chemicalWithoutSubmaps = {
      ...CHEMICALS_PIN.pin,
      targetElements: [],
    };

    renderComponent(CHEMICALS_PIN.name, chemicalWithoutSubmaps, 'chemicals');

    expect(screen.queryByText('Available in submaps:')).toBeNull();
  });
  it('should not call setBounds if coordinates do not exist', () => {
    useVisiblePinsPolygonCoordinatesMock.mockImplementation(() => undefined);

    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const buttonCenterMapToPin = screen.getByTestId('center-to-pin');

    expect(buttonCenterMapToPin).toBeInTheDocument();

    act(() => {
      buttonCenterMapToPin.click();
    });

    expect(setBounds).not.toHaveBeenCalled();
  });
  it('should call setBounds if coordinates exist', () => {
    useVisiblePinsPolygonCoordinatesMock.mockImplementation(() => [
      [292, 333],
      [341, 842],
    ]);

    renderComponent(DRUGS_PIN.name, DRUGS_PIN.pin, 'drugs');

    const buttonCenterMapToPin = screen.getByTestId('center-to-pin');

    expect(buttonCenterMapToPin).toBeInTheDocument();

    act(() => {
      buttonCenterMapToPin.click();
    });

    expect(setBounds).toHaveBeenCalled();
  });
});
