import { setMapPosition, varyPositionZoom } from '@/redux/map/map.slice';
import { SetBoundsResult, useSetBounds } from '@/utils/map/useSetBounds';
import { useCallback } from 'react';
import { useDispatch } from 'react-redux';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { currentModelIdSelector, modelByIdSelector } from '@/redux/models/models.selectors';
import { DEFAULT_ZOOM } from '@/constants/map';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { mapDataLastPositionSelector } from '@/redux/map/map.selectors';
import { useVisibleBioEntitiesPolygonCoordinates } from './useVisibleBioEntitiesPolygonCoordinates';
import { MAP_ZOOM_IN_DELTA, MAP_ZOOM_OUT_DELTA } from '../MappAdditionalActions.constants';

interface UseAddtionalActionsResult {
  zoomIn(): void;
  zoomOut(): void;
  zoomInToBioEntities(): void;
}

export const useAddtionalActions = (): UseAddtionalActionsResult => {
  const dispatch = useDispatch();
  const setBounds = useSetBounds();
  const polygonCoordinates = useVisibleBioEntitiesPolygonCoordinates();
  const currentMapModelId = useAppSelector(currentModelIdSelector);
  const currentModel = useAppSelector(state => modelByIdSelector(state, currentMapModelId));
  const currentModelLastPostiion = useAppSelector(mapDataLastPositionSelector);

  const zoomInToBioEntities = (): SetBoundsResult | undefined => {
    if (polygonCoordinates) {
      return setBounds(polygonCoordinates);
    }

    if (currentModel) {
      const HALF = 2;
      const defaultPosition = {
        x: currentModel?.defaultCenterX ?? currentModel.width / HALF,
        y: currentModel?.defaultCenterY ?? currentModel.height / HALF,
        z: currentModel?.defaultZoomLevel ?? DEFAULT_ZOOM,
      };

      dispatch(setMapPosition(defaultPosition));

      if (currentModelLastPostiion.z !== defaultPosition.z) {
        PluginsEventBus.dispatchEvent('onZoomChanged', {
          modelId: currentMapModelId,
          zoom: defaultPosition.z,
        });
      }

      if (
        currentModelLastPostiion.x !== defaultPosition.x ||
        currentModelLastPostiion.y !== defaultPosition.y
      ) {
        PluginsEventBus.dispatchEvent('onCenterChanged', {
          modelId: currentMapModelId,
          x: defaultPosition.x,
          y: defaultPosition.y,
        });
      }
    }

    return undefined;
  };

  const varyZoomByDelta = useCallback(
    (delta: number) => {
      dispatch(varyPositionZoom({ delta }));
    },
    [dispatch],
  );

  return {
    zoomIn: () => varyZoomByDelta(MAP_ZOOM_IN_DELTA),
    zoomOut: () => varyZoomByDelta(MAP_ZOOM_OUT_DELTA),
    zoomInToBioEntities,
  };
};
