/* eslint-disable no-magic-numbers */
import { CONFIGURATION_INITIAL_STORE_MOCKS } from '@/redux/configuration/configuration.mock';
import { mapStateWithCurrentlySelectedMainMapFixture } from '@/redux/map/map.fixtures';
import { MODELS_DATA_MOCK_WITH_MAIN_MAP } from '@/redux/models/models.mock';
import { MOCKED_OVERLAY_BIO_ENTITY_RENDER } from '@/redux/overlayBioEntity/overlayBioEntity.mock';
import { OVERLAYS_PUBLIC_FETCHED_STATE_MOCK } from '@/redux/overlays/overlays.mock';
import { getReduxWrapperWithStore } from '@/utils/testing/getReduxWrapperWithStore';
import { renderHook } from '@testing-library/react';
import { useOverlayFeatures } from './useOverlayFeatures';

/**
 * mocks for useOverlayFeatures
 * are taken from helpers functions that are used inside useOverlayFeatures
 * point of the test is to test if all helper functions work correctly when combined together
 */

describe('useOverlayFeatures', () => {
  const { Wrapper } = getReduxWrapperWithStore({
    configuration: {
      ...CONFIGURATION_INITIAL_STORE_MOCKS,
    },
    overlayBioEntity: {
      overlaysId: [11, 12],
      data: {
        // overlayId
        11: {
          // modelId
          52: MOCKED_OVERLAY_BIO_ENTITY_RENDER,
          53: MOCKED_OVERLAY_BIO_ENTITY_RENDER,
        },
        12: {
          52: MOCKED_OVERLAY_BIO_ENTITY_RENDER,
          53: MOCKED_OVERLAY_BIO_ENTITY_RENDER,
        },
      },
    },
    overlays: {
      ...OVERLAYS_PUBLIC_FETCHED_STATE_MOCK,
    },
    map: {
      ...mapStateWithCurrentlySelectedMainMapFixture,
    },
    models: {
      ...MODELS_DATA_MOCK_WITH_MAIN_MAP,
    },
  });

  it('should return an array of features', () => {
    const {
      result: { current: features },
    } = renderHook(() => useOverlayFeatures(), {
      wrapper: Wrapper,
    });

    expect(features).toHaveLength(10);

    // type: rectangle
    expect(features[0].getGeometry()?.getCoordinates()).toEqual([
      [
        [-13149141, 18867005],
        [-13149141, 18881970],
        [-13143529, 18881970],
        [-13143529, 18867005],
        [-13149141, 18867005],
      ],
    ]);
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    expect(features[0].getStyle().getFill().getColor()).toBe('#FFFFFFcc');

    // type: line
    expect(features[7].getGeometry()?.getCoordinates()).toEqual([
      [
        [-13149141, 18867005],
        [-13149141, 18881970],
        [-13141659, 18881970],
        [-13141659, 18867005],
        [-13149141, 18867005],
      ],
    ]);
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    expect(features[7].getStyle().getFill().getColor()).toBe('#ff0000cc');
  });
});
