/* eslint-disable no-magic-numbers */
import { searchedBioEntitesSelectorOfCurrentMap } from '@/redux/bioEntity/bioEntity.selectors';
import { searchedChemicalsBioEntitesOfCurrentMapSelector } from '@/redux/chemicals/chemicals.selectors';
import { searchedDrugsBioEntitesOfCurrentMapSelector } from '@/redux/drugs/drugs.selectors';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import Feature from 'ol/Feature';
import { Geometry } from 'ol/geom';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useMemo } from 'react';
import { useSelector } from 'react-redux';
import { getBioEntitiesFeatures } from './getBioEntitiesFeatures';

export const useOlMapPinsLayer = (): VectorLayer<VectorSource<Feature<Geometry>>> => {
  const pointToProjection = usePointToProjection();
  const contentBioEntites = useSelector(searchedBioEntitesSelectorOfCurrentMap);
  const chemicalsBioEntities = useSelector(searchedChemicalsBioEntitesOfCurrentMapSelector);
  const drugsBioEntities = useSelector(searchedDrugsBioEntitesOfCurrentMapSelector);

  const bioEntityFeatures = useMemo(
    () =>
      [
        getBioEntitiesFeatures(contentBioEntites, { pointToProjection, type: 'bioEntity' }),
        getBioEntitiesFeatures(chemicalsBioEntities, { pointToProjection, type: 'chemicals' }),
        getBioEntitiesFeatures(drugsBioEntities, { pointToProjection, type: 'drugs' }),
      ].flat(),
    [contentBioEntites, drugsBioEntities, chemicalsBioEntities, pointToProjection],
  );

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features: [...bioEntityFeatures],
    });
  }, [bioEntityFeatures]);

  const pinsLayer = useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );

  return pinsLayer;
};
