import { z } from 'zod';
import { colorSchema } from './colorSchema';
import { glyphSchema } from './glyphSchema';
import { lineSchema } from './lineSchema';
import { modificationResiduesSchema } from './modificationResiduesSchema';
import { operatorSchema } from './operatorSchema';
import { productsSchema } from './products';
import { referenceSchema } from './referenceSchema';
import { structuralStateSchema } from './structuralStateSchema';
import { submodelSchema } from './submodelSchema';

export const bioEntitySchema = z.object({
  id: z.number(),
  stringType: z.string(),
  name: z.string(),
  elementId: z.string(),
  model: z.number(),
  references: z.array(referenceSchema),
  z: z.number(),
  notes: z.string(),
  symbol: z.string().nullable(),
  homodimer: z.number().optional(),
  nameX: z.number(),
  nameY: z.number(),
  nameWidth: z.number(),
  nameHeight: z.number(),
  nameVerticalAlign: z.string(),
  nameHorizontalAlign: z.string(),
  width: z.number(),
  height: z.number(),
  visibilityLevel: z.string(),
  transparencyLevel: z.string(),
  synonyms: z.array(z.string()),
  formerSymbols: z.array(z.string()),
  fullName: z.string().nullable(),
  compartmentName: z.string().nullable().optional(),
  abbreviation: z.string().nullable(),
  formula: z.string().nullable(),
  glyph: glyphSchema.nullable(),
  activity: z.boolean().optional(),
  structuralState: z.optional(structuralStateSchema.nullable()),
  hypothetical: z.boolean().nullable().optional(),
  boundaryCondition: z.boolean().optional(),
  constant: z.boolean().optional(),
  initialAmount: z.number().nullable().optional(),
  initialConcentration: z.number().nullable().optional(),
  charge: z.number().nullable().optional(),
  substanceUnits: z.string().nullable().optional(),
  onlySubstanceUnits: z.boolean().optional(),
  modificationResidues: z.optional(z.array(modificationResiduesSchema)),
  complex: z.number().nullable().optional(),
  compartment: z.number().nullable(),
  submodel: submodelSchema.nullable(),
  x: z.number(),
  y: z.number(),
  lineWidth: z.number().optional(),
  fontColor: colorSchema,
  fontSize: z.number(),
  fillColor: colorSchema,
  borderColor: colorSchema,
  smiles: z.optional(z.string()).nullable(),
  inChI: z.optional(z.string().nullable()),
  inChIKey: z.optional(z.string().nullable()),
  thickness: z.optional(z.number()),
  outerWidth: z.optional(z.number()),
  innerWidth: z.optional(z.number()),
  idReaction: z.optional(z.string()),
  reversible: z.optional(z.boolean()),
  mechanicalConfidenceScore: z.optional(z.boolean()),
  lowerBound: z.optional(z.boolean()),
  upperBound: z.optional(z.boolean()),
  subsystem: z.optional(z.string()),
  geneProteinReaction: z.optional(z.string()),
  kinetics: z.optional(z.null()),
  products: z.optional(z.array(productsSchema)),
  reactants: z.optional(z.array(productsSchema)),
  modifiers: z.optional(z.array(productsSchema)),
  processCoordinates: z.optional(z.null()),
  line: z.optional(lineSchema),
  operators: z.optional(z.array(operatorSchema)),
});
