/* eslint-disable no-magic-numbers */
import { axiosInstanceNewAPI } from '@/services/api/utils/axiosInstance';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { PROJECT_ID } from '@/constants';
import { ExportNetwork, ExportElements } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { exportNetworkchema, exportElementsSchema } from '@/models/exportSchema';
import { apiPath } from '../apiPath';
import { downloadFileFromBlob } from './export.utils';

type DownloadElementsBodyRequest = {
  columns: string[];
  submaps: number[];
  annotations: string[];
  includedCompartmentIds: number[];
  excludedCompartmentIds: number[];
};

export const downloadElements = createAsyncThunk(
  'export/downloadElements',
  async (data: DownloadElementsBodyRequest): Promise<void> => {
    const response = await axiosInstanceNewAPI.post<ExportElements>(
      apiPath.downloadElementsCsv(),
      data,
      {
        withCredentials: true,
      },
    );

    const isDataValid = validateDataUsingZodSchema(response.data, exportElementsSchema);

    if (isDataValid) {
      downloadFileFromBlob(response.data, `${PROJECT_ID}-elementExport.csv`);
    }
  },
);

type DownloadNetworkBodyRequest = {
  columns: string[];
  submaps: number[];
  annotations: string[];
  includedCompartmentIds: number[];
  excludedCompartmentIds: number[];
};

export const downloadNetwork = createAsyncThunk(
  'export/downloadNetwork',
  async (data: DownloadNetworkBodyRequest): Promise<void> => {
    const response = await axiosInstanceNewAPI.post<ExportNetwork>(
      apiPath.downloadNetworkCsv(),
      data,
      {
        withCredentials: true,
      },
    );

    const isDataValid = validateDataUsingZodSchema(response.data, exportNetworkchema);

    if (isDataValid) {
      downloadFileFromBlob(response.data, `${PROJECT_ID}-networkExport.csv`);
    }
  },
);
