import { OverlayBioEntityRender } from '@/types/OLrendering';
import { createSelector } from '@reduxjs/toolkit';
import { currentSearchedBioEntityId } from '../drawer/drawer.selectors';
import { currentModelIdSelector } from '../models/models.selectors';
import { overlaysDataSelector, overlaysIdsAndOrderSelector } from '../overlays/overlays.selectors';
import { rootSelector } from '../root/root.selectors';
import { calculateOvarlaysOrder } from './overlayBioEntity.utils';

export const overlayBioEntitySelector = createSelector(
  rootSelector,
  state => state.overlayBioEntity,
);

export const overlayBioEntityDataSelector = createSelector(
  overlayBioEntitySelector,
  overlayBioEntity => overlayBioEntity.data,
);

export const activeOverlaysIdSelector = createSelector(
  overlayBioEntitySelector,
  state => state.overlaysId,
);

export const overlayBioEntitiesForCurrentModelSelector = createSelector(
  overlayBioEntityDataSelector,
  activeOverlaysIdSelector,
  currentModelIdSelector,
  (data, activeOverlaysIds, currentModelId) => {
    const result: OverlayBioEntityRender[] = [];

    activeOverlaysIds.forEach(overlayId => {
      if (data[overlayId]?.[currentModelId]) {
        result.push(...data[overlayId][currentModelId]);
      }
    });

    return result;
  },
);

export const isOverlayActiveSelector = createSelector(
  [activeOverlaysIdSelector, (_, overlayId: number): number => overlayId],
  (overlaysId, overlayId) => overlaysId.includes(overlayId),
);

export const isOverlayLoadingSelector = createSelector(
  [overlayBioEntitySelector, (_, overlayId: number): number => overlayId],
  ({ overlaysId, data }, overlayId) =>
    overlaysId.includes(overlayId) && data[overlayId] && !Object.keys(data[overlayId]).length,
);

export const activeOverlaysSelector = createSelector(
  rootSelector,
  overlaysDataSelector,
  (state, overlaysData) =>
    overlaysData.filter(overlay => isOverlayActiveSelector(state, overlay.idObject)),
);

export const getOverlayOrderSelector = createSelector(
  overlaysIdsAndOrderSelector,
  activeOverlaysIdSelector,
  (overlaysIdsAndOrder, activeOverlaysIds) => {
    const activeOverlaysIdsAndOrder = overlaysIdsAndOrder.filter(({ idObject }) =>
      activeOverlaysIds.includes(idObject),
    );

    return calculateOvarlaysOrder(activeOverlaysIdsAndOrder);
  },
);

export const overlaysOpenedIdsSelector = createSelector(
  rootSelector,
  state => state.overlayBioEntity.overlaysId,
);

export const overlaysOpenedSelector = createSelector(
  overlaysDataSelector,
  overlaysOpenedIdsSelector,
  (data, ids) => data.filter(entity => ids.includes(entity.idObject)),
);

export const overlaysBioEntityForCurrentBioEntityAndCurrentModelSelector = createSelector(
  overlayBioEntitiesForCurrentModelSelector,
  currentSearchedBioEntityId,
  (data, currentBioEntityId) => data.filter(entity => entity.id === currentBioEntityId),
);
