import { QueryDataParams } from '@/types/query';
import { createSelector } from '@reduxjs/toolkit';
import { ZERO } from '@/constants/common';
import { mapDataSelector } from '../map/map.selectors';
import { perfectMatchSelector, searchValueSelector } from '../search/search.selectors';
import { activeOverlaysIdSelector } from '../overlayBioEntity/overlayBioEntity.selector';
import { activePluginsIdSelector } from '../plugins/plugins.selectors';

export const queryDataParamsSelector = createSelector(
  searchValueSelector,
  perfectMatchSelector,
  mapDataSelector,
  activeOverlaysIdSelector,
  activePluginsIdSelector,
  (
    searchValue,
    perfectMatch,
    { modelId, backgroundId, position },
    activeOverlaysId,
    activePluginsId,
  ): QueryDataParams => {
    const joinedSearchValue = searchValue.join(';');
    const shouldIncludeSearchValue = searchValue.length > ZERO && joinedSearchValue;

    const shouldIncludeOverlaysId = activeOverlaysId.length > ZERO;
    const shouldIncludePluginsId = activePluginsId.length > ZERO;

    const queryDataParams: QueryDataParams = {
      perfectMatch,
      modelId,
      backgroundId,
      ...position.last,
      ...(shouldIncludeSearchValue ? { searchValue: joinedSearchValue } : {}),
      ...(shouldIncludeOverlaysId ? { overlaysId: activeOverlaysId.join(',') } : {}),
      ...(shouldIncludePluginsId ? { pluginsId: activePluginsId.join(',') } : {}),
    };

    return queryDataParams;
  },
);
