import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { loginSchema } from '@/models/loginSchema';
import { sessionSchemaValid } from '@/models/sessionValidSchema';
import { Login, SessionValid } from '@/types/models';
import { apiPath } from '../apiPath';
import { closeModal } from '../modal/modal.slice';

export const login = createAsyncThunk(
  'user/login',
  async (credentials: { login: string; password: string }, { dispatch }) => {
    const searchParams = new URLSearchParams(credentials);
    const response = await axiosInstance.post<Login>(apiPath.postLogin(), searchParams, {
      withCredentials: true,
    });

    const isDataValid = validateDataUsingZodSchema(response.data, loginSchema);
    dispatch(closeModal());

    return isDataValid ? response.data : undefined;
  },
);

export const getSessionValid = createAsyncThunk('user/getSessionValid', async () => {
  const response = await axiosInstance.get<SessionValid>(apiPath.getSessionValid(), {
    withCredentials: true,
  });

  const isDataValid = validateDataUsingZodSchema(response.data, sessionSchemaValid);

  return isDataValid ? response.data.login : null;
});
