import { SearchParams } from './triggerSearch.types';
import { searchByQuery } from './searchByQuery';
import { searchByCoordinates } from './searchByCoordinates';

export async function triggerSearch(params: SearchParams): Promise<void> {
  if ('query' in params) {
    if (typeof params.query !== 'string') {
      throw new Error('Invalid query type. The query should be of string type');
    }
    searchByQuery(params.query, params.perfectSearch);
  } else {
    const areCoordinatesInvalidType =
      typeof params.coordinates !== 'object' || params.coordinates === null;
    const areCoordinatesMissingKeys = !('x' in params.coordinates) || !('y' in params.coordinates);
    const areCoordinatesValuesInvalid =
      typeof params.coordinates.x !== 'number' || typeof params.coordinates.y !== 'number';

    if (areCoordinatesInvalidType || areCoordinatesMissingKeys || areCoordinatesValuesInvalid) {
      throw new Error('Invalid coordinates type or values');
    }

    if (typeof params.modelId !== 'number') {
      throw new Error('Invalid model id type. The model should be of number type');
    }

    searchByCoordinates(params.coordinates, params.modelId);
  }
}
