import { BioEntityContent, Chemical, CreatedOverlay, Drug, MapOverlay } from '@/types/models';
import { dispatchEvent } from './pluginsEventBus';

export type BackgroundEvents = 'onBackgroundOverlayChange';
export type OverlayEvents =
  | 'onAddDataOverlay'
  | 'onRemoveDataOverlay'
  | 'onShowOverlay'
  | 'onHideOverlay';
export type SubmapEvents =
  | 'onSubmapOpen'
  | 'onSubmapClose'
  | 'onZoomChanged'
  | 'onCenterChanged'
  | 'onBioEntityClick';
export type SearchEvents = 'onSearch';

export type Events = OverlayEvents | BackgroundEvents | SubmapEvents | SearchEvents;

export type ZoomChanged = {
  zoom: number;
  modelId: number;
};

export type CenteredCoordinates = {
  modelId: number;
  x: number;
  y: number;
};

export type ClickedBioEntity = {
  id: number;
  type: string;
  modelId: number;
};

export type SearchDataBioEntity = {
  type: 'bioEntity';
  searchValues: string[];
  results: BioEntityContent[][];
};

export type SearchDataDrugs = {
  type: 'drugs';
  searchValues: string[];
  results: Drug[][];
};

export type SearchDataChemicals = {
  type: 'chemicals';
  searchValues: string[];
  results: Chemical[][];
};

export type SearchData = SearchDataBioEntity | SearchDataDrugs | SearchDataChemicals;

export type EventsData =
  | CreatedOverlay
  | number
  | MapOverlay
  | ZoomChanged
  | CenteredCoordinates
  | ClickedBioEntity
  | SearchData;

export type PluginsEventBusType = {
  events: {
    hash: string;
    type: Events;
    callback: (data: unknown) => void;
  }[];
  addListener: (hash: string, type: Events, callback: (data: unknown) => void) => void;
  removeListener: (hash: string, type: Events, callback: unknown) => void;
  removeAllListeners: (hash: string) => void;
  dispatchEvent: typeof dispatchEvent;
};
