import { PLUGINS_CONTENT_ELEMENT_ATTR_NAME, PLUGINS_CONTENT_ELEMENT_ID } from '@/constants/plugins';
import { registerPlugin } from '@/redux/plugins/plugins.thunks';
import { store } from '@/redux/store';
import md5 from 'crypto-js/md5';
import type { PluginsManagerType } from './pluginsManager.types';
import { configurationMapper } from './pluginsManager.utils';
import { triggerSearch } from './map/triggerSearch';
import { PluginsEventBus } from './pluginsEventBus';

export const PluginsManager: PluginsManagerType = {
  hashedPlugins: {},
  setHashedPlugin({ pluginUrl, pluginScript }) {
    const hash = md5(pluginScript).toString();

    PluginsManager.hashedPlugins[pluginUrl] = hash;

    return hash;
  },
  init() {
    window.minerva = {
      plugins: {
        registerPlugin: PluginsManager.registerPlugin,
      },
      map: {
        triggerSearch,
      },
    };

    const unsubscribe = store.subscribe(() => {
      const configurationStore = store.getState().configuration.main.data;

      if (configurationStore) {
        const configuration = configurationMapper(configurationStore);

        window.minerva = {
          ...window.minerva,
          configuration,
        };
      }
    });

    return unsubscribe;
  },

  registerPlugin({ pluginName, pluginVersion, pluginUrl }) {
    const hash = PluginsManager.hashedPlugins[pluginUrl];

    store.dispatch(
      registerPlugin({
        hash,
        isPublic: false,
        pluginName,
        pluginUrl,
        pluginVersion,
      }),
    );

    const element = PluginsManager.createAndGetPluginContent({ hash });

    return {
      element,
      events: {
        addListener: PluginsEventBus.addListener.bind(this, hash),
        removeListener: PluginsEventBus.removeListener.bind(this, hash),
        removeAllListeners: PluginsEventBus.removeAllListeners.bind(this, hash),
      },
    };
  },
  createAndGetPluginContent({ hash }) {
    const element = document.createElement('div');
    element.setAttribute(PLUGINS_CONTENT_ELEMENT_ATTR_NAME, hash);

    const wrapper = document.querySelector(`#${PLUGINS_CONTENT_ELEMENT_ID}`);
    wrapper?.append(element);

    return element;
  },
  removePluginContent({ hash }) {
    const elements = document.querySelectorAll(
      `div[${PLUGINS_CONTENT_ELEMENT_ATTR_NAME}="${hash}"]`,
    );

    elements.forEach(element => element.remove());
  },
};
